@@ -2,8 +2,12 @@ class RecursionError(Exception):
     def __init__(self, *args, **kwargs):
         Exception.__init__(self, *args, **kwargs)
 
-class FailedIntegrationError(Exception):
+class FailedIntegration(Exception):
+    def __init__(self, *args, **kwargs):
+        Exception.__init__(self, *args, **kwargs)
+
+class FailedToMeetTolerances(Exception):
     def __init__(self, *args, **kwargs):
         Exception.__init__(self, *args, **kwargs)
         
-__all__ = ["RecursionError", "FailedIntegrationError"]
+__all__ = ["RecursionError", "FailedIntegration", "FailedToMeetTolerances"]
